"use client";

import { zodResolver } from "@hookform/resolvers/zod";
import { useSession } from "next-auth/react";
import { useState } from "react";
import { useForm } from "react-hook-form";
import * as z from "zod";

import { Button } from "@/components/ui/button";
import {
  Form,
  FormControl,
  FormDescription,
  FormField,
  FormItem,
  FormMessage,
} from "@/components/ui/form";

import { Textarea } from "@/components/ui/textarea";
import { toast } from "@/components/ui/use-toast";

import { IGweet } from "@/components/gweets";
import { Icons } from "@/components/icons";
import { UserAvatar } from "@/components/user-avatar";

import { Card } from "@/components/ui/card";
import { useCreateGweet } from "../hooks/use-create-gweet";

const FormSchema = z.object({
  gweet: z
    .string()
    .min(1, { message: "Come on post something...", })
    .max(1000, { message: "Gweets cannot be more that 1000 characters.", }),
})

export const CreateGweet = ({
  parent_gweet,
  replyToUserId,
  replyToGweetId,
  placeholder,
  isComment = false,
}: {
  parent_gweet?: IGweet | null;
  replyToUserId?: string | null;
  replyToGweetId?: string | null;
  placeholder?: string | null;
  isComment?: boolean;
}) => {
  const [isGweetLoading, setIsGweetLoading] = useState<boolean>(false);

  const { data: session } = useSession();
  const mutation = useCreateGweet();

  const form = useForm<z.infer<typeof FormSchema>>({
    resolver: zodResolver(FormSchema),
  })

  function onGweet(data: z.infer<typeof FormSchema>) {
    if (!session) return null;

    setIsGweetLoading(true);

    mutation.mutate({
      content: data.gweet,
      userId: session?.user?.id,
      replyToUserId,
      replyToGweetId,
    })

    toast({
      description: "Your gweet was send.",
    })

    setIsGweetLoading(false);
    form.setValue('gweet', '');
  }

  if (!session) return null;

  return (
    <>
      <div className="grid grid-cols-2 gap-11 p-4">
        {parent_gweet && (
          <div className="grid place-items-center grid-rows-2 gap-1">
            <UserAvatar
              user={{ username: parent_gweet?.user?.username, image: parent_gweet?.user?.image || null }}
            />
            <div className="bg-gray-300 h-full w-px"></div>
          </div>
        )}
        <div className="flex flex-col">
          {parent_gweet && (
            <>
              <div className="flex gap-1">
                <span className="text-secondary text-sm font-medium truncate hover:underline">
                  {parent_gweet?.user?.name}
                </span>
                <span className="text-tertiary text-sm truncate">
                  @{parent_gweet?.user?.email?.split("@")[0]}
                </span>
                <span className="text-tertiary">·</span>
              </div>
              <div>
                {parent_gweet?.content && (
                  <div className="text">{parent_gweet?.content}</div>
                )}
              </div>
            </>
          )}

          {replyToUserId && !isComment && (
            <div className={`${!parent_gweet ? 'ml-16' : ''} flex items-center gap-1 cursor-pointer`}>
              <span className="text-tertiary truncate">Replying to</span>
              <span className="text-primary truncate">
                @{replyToUserId}
              </span>
            </div>
          )}
        </div>
      </div>

      <div className="relative">
        <Form {...form}>
          <form onSubmit={form.handleSubmit(onGweet)} className="space-y-6">
            <Card className="p-3">
              <FormField
                control={form.control}
                name="gweet"
                render={({ field }) => (
                  <FormItem className="flex items-start">
                    <UserAvatar
                      className="mr-3"
                      user={{ username: session?.user.username || null, image: session?.user?.image || null }}
                    />
                    <div className="flex-grow">
                      <FormControl>
                        <Textarea
                          placeholder={placeholder || "What's on your mind?"}
                          className="resize-none"
                          disabled={isGweetLoading || !session.user}
                          {...field}
                        />
                      </FormControl>
                      {!isComment ?
                        <FormDescription className="pt-3">
                          Your gweets will be public, and everyone can see them.
                        </FormDescription>
                        : null
                      }
                      <FormMessage />
                    </div>
                  </FormItem>
                )}
              />
            </Card>

            <div className="flex justify-end">
              <Button type="submit" size="lg" disabled={isGweetLoading || !session.user}>
                {isComment ? 'Reply' : 'Gweet'}
              </Button>
            </div>
          </form>
        </Form>
      </div>
    </>
  );
};
